/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
#include <errno.h>
#include <stdio.h>
#include <sys/ioctl.h>

#include "hpdi32_diphase2.h"



/******************************************************************************
*
*	Function:	hpdi32_dp2_board_cc_d5_d6_disable_get
*
*	Purpose:
*
*		Report the current setting that disables driving of Cable Command
*		signals D5 and D6.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		disable	Are D5 and D6 outputs disabled? 0 = no, 1 = yes
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_board_cc_d5_d6_disable_get(int fd, int* disable)
{
	__u32	bcr;
	int		err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

	if (disable)
		disable[0]	= (bcr & HPDI32_GSC_BCR_CC_D5_D6_DISABLE) ? 1 : 0;

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_board_cc_d5_d6_disable_set
*
*	Purpose:
*
*		Update the current setting that disables driving of Cable Command
*		signals D5 and D6.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		disable	Disable D5 and D6 outputs? <0 = ignore, 0 = no, 1 = yes
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_board_cc_d5_d6_disable_set(int fd, int disable)
{
	int		err	= 0;
	__u32	mask;
	__u32	value;

	if (disable >= 0)
	{
		mask	= HPDI32_GSC_BCR_CC_D5_D6_DISABLE;
		value	= disable ? mask : 0;
		err		= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, value, mask);
	}

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_board_loop_back_get
*
*	Purpose:
*
*		Report the current loop back setting.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		enable	Is loop back enabled? 0 = no, 1 = yes
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_board_loop_back_get(int fd, int* enable)
{
	__u32	bcr;
	int		err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

	if (enable)
		enable[0]	= (bcr & HPDI32_GSC_BCR_LOOP_BACK) ? 1 : 0;

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_board_loop_back_set
*
*	Purpose:
*
*		Update the current loop back setting.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		enable	Enable loop back mode? <0 = ignore, 0 = disable, >0 = enable
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_board_loop_back_set(int fd, int enable)
{
	int		err	= 0;
	__u32	mask;
	__u32	value;

	if (enable >= 0)
	{
		mask	= HPDI32_GSC_BCR_LOOP_BACK;
		value	= enable ? mask : 0;
		err		= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, value, mask);
	}

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_board_reset
*
*	Purpose:
*
*		Perform a board reset.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_board_reset(int fd)
{
	int		err;
	__u32	mask;
	__u32	value;

	// The added mask bits are those not cleared by the reset bit.
	mask	= HPDI32_GSC_BCR_BOARD_RESET
			| HPDI32_GSC_BCR_CC_D5_D6_DISABLE
			| HPDI32_GSC_BCR_LOOP_BACK
			| HPDI32_GSC_BCR_PO_07_00_ENABLE
			| HPDI32_GSC_BCR_PO_15_08_ENABLE
			| HPDI32_GSC_BCR_PO_23_16_ENABLE
			| HPDI32_GSC_BCR_PO_31_24_ENABLE
			| HPDI32_GSC_BCR_RX_ALL_DATA
			| HPDI32_GSC_BCR_RX_BIG_ENDIAN
			| HPDI32_GSC_BCR_RX_ENABLE
			| HPDI32_GSC_BCR_RX_HDR_SIZE_14
			| HPDI32_GSC_BCR_TX_BIG_ENDIAN
			| HPDI32_GSC_BCR_TX_ENABLE
			| HPDI32_GSC_BCR_TX_PD_ENABLE
			| HPDI32_GSC_BCR_TX_PD_LEVEL
			| HPDI32_GSC_BCR_TX_RAW_OUTPUT
			| HPDI32_GSC_BCR_TX_START;
	value	= HPDI32_GSC_BCR_BOARD_RESET;

	err	= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, value, mask);
	return(err);
}



