/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
#include <errno.h>
#include <stdio.h>
#include <sys/ioctl.h>

#include "hpdi32_diphase2.h"



/******************************************************************************
*
*	Function:	hpdi32_dp2_gpio_dir_get
*
*	Purpose:
*
*		Retrieve the direction of the GPIO ports.
*
*	Arguments:
*
*		fd			The handle used to access the device.
*
*		out_hwhb	Is the High Word, High Byte an output?
*					This refers to bits 31 to 24.
*					This will be one if it is an output, and zero if an input.
*
*		out_hwlb	Is the High Word, Low Byte an output?
*					This refers to bits 23 to 16.
*					This will be one if it is an output, and zero if an input.
*
*		out_lwhb	Is the Low Word, High Byte an output?
*					This refers to bits 15 to 8.
*					This will be one if it is an output, and zero if an input.
*
*		out_lwlb	Is the Low Word, Low Byte an output?
*					This refers to bits 7 to 0.
*					This will be one if it is an output, and zero if an input.
*
*	Returned:
*
*		0			All went well.
*		!0			The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_gpio_dir_get(
	int		fd,
	int*	out_hwhb,
	int*	out_hwlb,
	int*	out_lwhb,
	int*	out_lwlb)
{
	__u32	bcr;
	int		err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

	if (out_hwhb)
		out_hwhb[0]	= (bcr & HPDI32_GSC_BCR_PO_31_24_ENABLE) ? 1 : 0;

	if (out_hwlb)
		out_hwlb[0]	= (bcr & HPDI32_GSC_BCR_PO_23_16_ENABLE) ? 1 : 0;

	if (out_lwhb)
		out_lwhb[0]	= (bcr & HPDI32_GSC_BCR_PO_15_08_ENABLE) ? 1 : 0;

	if (out_lwlb)
		out_lwlb[0]	= (bcr & HPDI32_GSC_BCR_PO_07_00_ENABLE) ? 1 : 0;

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_gpio_dir_set
*
*	Purpose:
*
*		Set the direction of the GPIO ports.
*
*	Arguments:
*
*		fd			The handle used to access the device.
*
*		out_hwhb	Set the High Word, High Byte to output?
*					This refers to bits 31 to 24.
*					<0 = ignore, 0 = input, >0 = output
*
*		out_hwlb	Set the High Word, Low Byte to output?
*					This refers to bits 23 to 16.
*					<0 = ignore, 0 = input, >0 = output
*
*		out_lwhb	Set the Low Word, High Byte to output?
*					This refers to bits 15 to 8.
*					<0 = ignore, 0 = input, >0 = output
*
*		out_lwlb	Set the Low Word, Low Byte to output?
*					This refers to bits 7 to 0.
*					<0 = ignore, 0 = input, >0 = output
*
*	Returned:
*
*		0			All went well.
*		!0			The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_gpio_dir_set(
	int	fd,
	int	out_hwhb,
	int	out_hwlb,
	int	out_lwhb,
	int	out_lwlb)
{
	int		err;
	__u32	mask	= 0;
	__u32	value	= 0;

	if (out_hwhb >= 0)
	{
		mask	|= HPDI32_GSC_BCR_PO_31_24_ENABLE;

		if (out_hwhb)
			value	|= HPDI32_GSC_BCR_PO_31_24_ENABLE;
	}

	if (out_hwlb >= 0)
	{
		mask	|= HPDI32_GSC_BCR_PO_23_16_ENABLE;

		if (out_hwlb)
			value	|= HPDI32_GSC_BCR_PO_23_16_ENABLE;
	}

	if (out_lwhb >= 0)
	{
		mask	|= HPDI32_GSC_BCR_PO_15_08_ENABLE;

		if (out_lwhb)
			value	|= HPDI32_GSC_BCR_PO_15_08_ENABLE;
	}

	if (out_lwlb >= 0)
	{
		mask	|= HPDI32_GSC_BCR_PO_07_00_ENABLE;

		if (out_lwlb)
			value	|= HPDI32_GSC_BCR_PO_07_00_ENABLE;
	}

	err	= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, value, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_gpio_mod
*
*	Purpose:
*
*		Perform a read-modify-write to the value that appears as the GPIO
*		cable signal output source.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		value	The value to apply to the data output register.
*
*		mask	The set of bits to modify. If a bit is set, then that bit
*				will be updated with the correspong "value" bit. If a bit is
*				zero, the the corresponding output bit will not be modified.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_gpio_mod(int fd, __u32 value, __u32 mask)
{
	int	err;

	err	= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_TXODPR, value, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_gpio_read
*
*	Purpose:
*
*		Read the signal levels at the GPIO cable signals.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		value	The value read from the device is stored here.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_gpio_read(int fd, __u32* value)
{
	int	err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_RXIDPR, value);
	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_gpio_write
*
*	Purpose:
*
*		Write the given value to appear at the GPIO cable signals. Only those
*		configured as output will actually appear at the cable interface.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		value	The value to apply to the data output register.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_gpio_write(int fd, __u32 value)
{
	int	err;

	err	= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXODPR, value);
	return(err);
}



