/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "hpdi32_diphase2.h"



/******************************************************************************
*
*	Function:	hpdi32_dp2_rx_config_get
*
*	Purpose:
*
*		Retrieve the current receiver configuration.
*
*	Arguments:
*
*		fd			The handle used to access the device.
*
*		enable		Is the receiver enabled? 0 = No, 1 = yes
*
*		big_e		Is the receiver configured for Big Enadian?
*					0 = Little Endian, 1 = Big Endian
*
*		hdr_size	Use 14 byte headers?
*					0 = 10 byte headers, 1 = 14 byte headers
*
*		all_data	Receive all data until the line goes idle?
*					0 = no, just till the message ends, 1 = yes
*
*	Returned:
*
*		0			All went well.
*		!0			The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_rx_config_get(
	int		fd,
	int*	enable,
	int*	big_e,
	int*	hdr_size,
	int*	all_data)
{
	__u32	bcr;
	int		err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

	if (enable)
		enable[0]	= (bcr & HPDI32_GSC_BCR_RX_ENABLE) ? 1 : 0;

	if (big_e)
		big_e[0]	= (bcr & HPDI32_GSC_BCR_RX_BIG_ENDIAN) ? 1 : 0;

	if (hdr_size)
		hdr_size[0]	= (bcr & HPDI32_GSC_BCR_RX_HDR_SIZE_14) ? 1 : 0;

	if (all_data)
		all_data[0]	= (bcr & HPDI32_GSC_BCR_RX_ALL_DATA) ? 1 : 0;

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_rx_config_set
*
*	Purpose:
*
*		Update the current receiver configuration.
*
*	Arguments:
*
*		fd			The handle used to access the device.
*
*		enable		Enable the receiver? <0 = ignore, 0 = disable, >0 = enable
*
*		big_e		Select Big Endian Rx ordering?
*					<0 = ignore, 0 = Little Endian, >0 = Big Endian
*
*		hdr_size	Use 14 byte headers?
*					<0 = ignore, 0 = 10 byte headers, >0 = 14 byte headers
*
*		all_data	Receive all data until the line goes idle?
*					<0 = ignore, 0 = no, just till the message ends, >0 = yes
*
*	Returned:
*
*		0			All went well.
*		!0			The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_rx_config_set(
	int	fd,
	int	enable,
	int	big_e,
	int	hdr_size,
	int	all_data)
{
	int		err;
	__u32	mask	= 0;
	__u32	value	= 0;

	if (enable >= 0)
	{
		mask	|= HPDI32_GSC_BCR_RX_ENABLE;

		if (enable)
			value	|= HPDI32_GSC_BCR_RX_ENABLE;
	}

	if (big_e >= 0)
	{
		mask	|= HPDI32_GSC_BCR_RX_BIG_ENDIAN;

		if (big_e)
			value	|= HPDI32_GSC_BCR_RX_BIG_ENDIAN;
	}

	if (hdr_size >= 0)
	{
		mask	|= HPDI32_GSC_BCR_RX_HDR_SIZE_14;

		if (hdr_size)
			value	|= HPDI32_GSC_BCR_RX_HDR_SIZE_14;
	}

	if (all_data >= 0)
	{
		mask	|= HPDI32_GSC_BCR_RX_ALL_DATA;

		if (all_data)
			value	|= HPDI32_GSC_BCR_RX_ALL_DATA;
	}

	err	= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, value, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_rx_fifo_almost_get
*
*	Purpose:
*
*		Retrieve the Almost settings of the Rx FIFO.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		af		The Almost Full setting is recorded here.
*
*		ae		The Almost Empty setting is recorded here.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_rx_fifo_almost_get(int fd, int* af, int* ae)
{
	int		err;
	__u32	rar	= 0xFFFFFFFF;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_RAR, &rar);

	if (af)
		af[0]	= HPDI32_GSC_RAR_DECODE_FULL(rar);

	if (ae)
		ae[0]	= HPDI32_GSC_RAR_DECODE_EMPTY(rar);

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_rx_fifo_almost_set
*
*	Purpose:
*
*		Update the Almost settings of the Rx FIFO.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		af		The Almost Full setting to apply.
*				<0 = ignore, >= 0 apply
*
*		ae		The Almost Empty setting to apply.
*				<0 = ignore, >= 0 apply
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_rx_fifo_almost_set(int fd, int af, int ae)
{
	int		err	= 0;
	__u32	rar;

	if ((af > 0xFFFF) || (ae > 0xFFFF))
		err	= EINVAL;

	if ((err) || ((af >= 0) && (ae >= 0)))
		;
	else
		err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_RAR, &rar);

	if (af < 0)
		af	= HPDI32_GSC_RAR_DECODE_FULL(rar);

	if (ae < 0)
		ae	= HPDI32_GSC_RAR_DECODE_EMPTY(rar);

	rar	= HPDI32_GSC_RAR_ENCODE(af, ae);

	if (err == 0)
	{
		err		= hpdi32_dp2_reg_write(fd, HPDI32_GSC_RAR, rar);

		if (err)
			hpdi32_dp2_rx_fifo_reset(fd);
		else
			err	= 	hpdi32_dp2_rx_fifo_reset(fd);
	}

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_rx_fifo_reset
*
*	Purpose:
*
*		Reset the Rx FIFO.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_rx_fifo_reset(int fd)
{
	int		err;

	err	= hpdi32_dp2_reg_mod(	fd,
								HPDI32_GSC_BCR,
								~0L,
								HPDI32_GSC_BCR_RX_FIFO_RESET);
	usleep(1000);	// Wait for the reset to complete.
	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_rx_accum_msg_size
*
*	Purpose:
*
*		Retrieve the recorded accumulated size of received messages. This will
*		report the current size so far if a message is in reception. The count
*		is reset to zero on the rising edge of the BCR Rx Enable bit. The count
*		stops incrementing when the receiver is disabled.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		size	The size, in bytes, is recorded here.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_rx_accum_msg_size(int fd, __u32* size)
{
	int	err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_RXMLSR, size);
	return(err);
}



