/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "hpdi32_diphase2.h"



/******************************************************************************
*
*	Function:	hpdi32_dp2_tx_config_get
*
*	Purpose:
*
*		Retrieve the current transmitter configuration.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		enable	Is the transmitter enabled? 0 = No, 1 = yes
*
*		start	Has the transmitter been started? 0 = No, 1 = yes
*
*		big_e	Is the transmitter configured for Big Enadian?
*				0 = Little Endian, 1 = Big Endian
*
*		raw		The raw data transmission mode is recorded here. 0 = No, 1 = Yes
*
*		length	The programmed message length is recorded here.
*				This value is in 16-bit words.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_tx_config_get(
	int		fd,
	int*	enable,
	int*	start,
	int*	big_e,
	int*	raw,
	__u32*	length)
{
	__u32	bcr;
	int		err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

	if (err == 0)
		err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXMLCR, length);

	if (enable)
		enable[0]	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;

	if (start)
		start[0]	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;

	if (big_e)
		big_e[0]	= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;

	if (raw)
		raw[0]		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_tx_config_set
*
*	Purpose:
*
*		Update the current transmitter configuration.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		enable	Enable the transmitter? <0 = ignore, 0 = disable, >0 = enable
*
*		start	Start the transmitter? <0 = ignore, 0 = no, >0 = yes
*
*		big_e	Select Big Endian Tx ordering?
*				<0 = ignore, 0 = Little Endian, >0 = Big Endian
*
*		raw		Output raw, uncoded data? <0 = ignore, 0 = no, >0 = yes
*
*		length	This is the desired transmit message length in 16-bit words.
*				<0 = ignore, 0 = length limit ignored, >0 apply this setting
*				The length can be set to any desired value, but values outside
*				the range of 0 to 0x7FFFFFFF must be applied by direct register
*				writes to the Tx Message Length Control Register.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_tx_config_set(
	int		fd,
	int		enable,
	int		start,
	int		big_e,
	int		raw,
	__s32	length)
{
	int		err		= 0;
	__u32	mask	= 0;
	__u32	value	= 0;

	if (enable >= 0)
	{
		mask	|= HPDI32_GSC_BCR_TX_ENABLE;

		if (enable)
			value	|= HPDI32_GSC_BCR_TX_ENABLE;
	}

	if (big_e >= 0)
	{
		mask	|= HPDI32_GSC_BCR_TX_BIG_ENDIAN;

		if (big_e)
			value	|= HPDI32_GSC_BCR_TX_BIG_ENDIAN;
	}

	if (raw >= 0)
	{
		mask	|= HPDI32_GSC_BCR_TX_RAW_OUTPUT;

		if (raw)
			value	|= HPDI32_GSC_BCR_TX_RAW_OUTPUT;
	}

	if (length >= 0)
		err	= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXMLCR, length);

	if (err == 0)
		err	= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, value, mask);

	if ((err == 0) && (start >= 0))
	{
		mask	= HPDI32_GSC_BCR_TX_START;
		value	= start ? HPDI32_GSC_BCR_TX_START : 0;
		err		= hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, value, mask);
	}

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_tx_fifo_almost_get
*
*	Purpose:
*
*		Retrieve the Almost settings of the Tx FIFO.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		af		The Almost Full setting is recorded here.
*
*		ae		The Almost Empty setting is recorded here.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_tx_fifo_almost_get(int fd, int* af, int* ae)
{
	int		err;
	__u32	tar	= 0xFFFFFFFF;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);

	if (af)
		af[0]	= HPDI32_GSC_TAR_DECODE_FULL(tar);

	if (ae)
		ae[0]	= HPDI32_GSC_TAR_DECODE_EMPTY(tar);

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_tx_fifo_almost_set
*
*	Purpose:
*
*		Update the Almost settings of the Tx FIFO.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		af		The Almost Full setting to apply.
*				<0 = ignore, >= 0 apply
*
*		ae		The Almost Empty setting to apply.
*				<0 = ignore, >= 0 apply
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_tx_fifo_almost_set(int fd, int af, int ae)
{
	int		err	= 0;
	__u32	tar;

	if ((af > 0xFFFF) || (ae > 0xFFFF))
		err	= EINVAL;

	if ((err) || ((af >= 0) && (ae >= 0)))
		;
	else
		err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);

	if (af < 0)
		af	= HPDI32_GSC_TAR_DECODE_FULL(tar);

	if (ae < 0)
		ae	= HPDI32_GSC_TAR_DECODE_EMPTY(tar);

	tar	= HPDI32_GSC_TAR_ENCODE(af, ae);

	if (err == 0)
	{
		err		= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TAR, tar);

		if (err)
			hpdi32_dp2_tx_fifo_reset(fd);
		else
			err	= 	hpdi32_dp2_tx_fifo_reset(fd);
	}

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_tx_fifo_reset
*
*	Purpose:
*
*		Reset the Tx FIFO.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_tx_fifo_reset(int fd)
{
	int	err;

	err	= hpdi32_dp2_reg_mod(	fd,
								HPDI32_GSC_BCR,
								~0L,
								HPDI32_GSC_BCR_TX_FIFO_RESET);
	usleep(1000);	// Wait for the reset to complete.
	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_tx_last_msg_size
*
*	Purpose:
*
*		Retrieve the recorded size of the last transmitted message.
*		This will report the current size so far if a message is in
*		transmission. The count is reset to zero on the rising edge of the
*		BCR Tx Start bit. The count stops incrementing when the message
*		ends, regardless of what causes the message to end.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		size	The size, in bytes, is recorded here.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_tx_last_msg_size(int fd, __u32* size)
{
	int	err;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXMLSR, size);
	return(err);
}



